#include <QvState.h>

#if defined (WIN16) || defined (WIN32)   
    #include "windows.h"
#endif
    
const char *QvState::stackNames[NumStacks] = {
    "Camera",
    "Coordinate3",
    "FontStyle",
    "Light",
    "MaterialBinding",
    "Material",
    "NormalBinding",
    "Normal",
    "ShapeHints",
    "Texture2",
    "Texture2Transformation",
    "TextureCoordinate2",
    "Transformation",
    "BaseColor",
};

QvState::QvState()
{
    stacks = new QvElement * [NumStacks];

    for (int i = 0; i < NumStacks; i++)
    stacks[i] = NULL;

    depth = 0;
}

QvState::~QvState()
{
    while (depth > 0)
    pop();

    delete [] stacks;
}

void
QvState::addElement(StackIndex stackIndex, QvElement *elt)
{
    elt->depth = depth;
    elt->next = stacks[stackIndex];
    stacks[stackIndex] = elt;
}

void
QvState::push()
{
    depth++;
}

void
QvState::pop()
{
    depth--;

    for (int i = 0; i < NumStacks; i++)
    while (stacks[i] != NULL && stacks[i]->depth > depth)
        popElement((StackIndex) i);
}

void
QvState::popElement(StackIndex stackIndex)
{
    QvElement *elt = stacks[stackIndex];
    stacks[stackIndex] = elt->next;
    delete elt;
}

void
QvState::print()
{

#if defined (WIN16) || defined (WIN32)   
    char    szStringSpace[1024] ;                        
#endif
    
#if defined (WIN16) || defined (WIN32)   
    MessageBox(NULL, "Traversal state:\n", "QvState::print",MB_ICONINFORMATION) ;
#else
    printf("Traversal state:\n");
#endif    
    
    for (int i = 0; i < NumStacks; i++) {

#if defined (WIN16) || defined (WIN32)
    sprintf(szStringSpace,"\tStack [%2d] (%s):\n", i, stackNames[i]); 
    MessageBox(NULL, szStringSpace, "QvState::print",MB_ICONINFORMATION) ;
#else
    printf("\tStack [%2d] (%s):\n", i, stackNames[i]);
#endif    

    if (stacks[i] == NULL)
#if defined (WIN16) || defined (WIN32)
        MessageBox(NULL, "\t\tNULL\n", "QvState::print",MB_ICONINFORMATION) ;
#else
        printf("\t\tNULL\n");
#endif    

    else
        for (QvElement *elt = stacks[i]; elt != NULL; elt = elt->next)
        elt->print();
    }
}
